; 
; Exports of file MLANG.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY MLANG.dll
EXPORTS
IsConvertINetStringAvailable
ConvertINetString
ConvertINetUnicodeToMultiByte
ConvertINetMultiByteToUnicode
ConvertINetReset
DllCanUnloadNow
DllGetClassObject
DllRegisterServer
DllUnregisterServer
GetGlobalFontLinkObject
LcidToRfc1766A
LcidToRfc1766W
Rfc1766ToLcidA
Rfc1766ToLcidW
