;
; Definition file of mspatcha.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "mspatcha.dll"
EXPORTS
ApplyPatchToFileA
ApplyPatchToFileByBuffers
ApplyPatchToFileByHandles
ApplyPatchToFileByHandlesEx
ApplyPatchToFileExA
ApplyPatchToFileExW
ApplyPatchToFileW
GetFilePatchSignatureA
GetFilePatchSignatureByBuffer
GetFilePatchSignatureByHandle
GetFilePatchSignatureW
NormalizeFileForPatchSignature
TestApplyPatchToFileA
TestApplyPatchToFileByBuffers
TestApplyPatchToFileByHandles
TestApplyPatchToFileW
