;
; Definition file of dsrole.dll
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "dsrole.dll"
EXPORTS
DsRoleAbortDownlevelServerUpgrade@16
DsRoleCancel@8
DsRoleDcAsDc@68
DsRoleDcAsReplica@12
DsRoleDemoteDc@44
DsRoleDnsNameToFlatName@16
DsRoleFreeMemory@4
DsRoleGetDatabaseFacts@24
DsRoleGetDcOperationProgress@12
DsRoleGetDcOperationResults@12
DsRoleGetPrimaryDomainInformation@12
DsRoleIfmHandleFree@8
DsRoleServerSaveStateForUpgrade@4
DsRoleUpgradeDownlevelServer@48
