;
; Definition file of HIDPARSE.SYS
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "HIDPARSE.SYS"
EXPORTS
HidP_FreeCollectionDescription@4
HidP_GetButtonCaps@16
HidP_GetCaps@8
HidP_GetCollectionDescription@16
HidP_GetData@24
HidP_GetExtendedAttributes@20
HidP_GetLinkCollectionNodes@12
HidP_GetScaledUsageValue@32
HidP_GetSpecificButtonCaps@28
HidP_GetSpecificValueCaps@28
HidP_GetUsageValue@32
HidP_GetUsageValueArray@36
HidP_GetUsages@32
HidP_GetUsagesEx@28
HidP_GetValueCaps@16
HidP_InitializeReportForID@20
HidP_MaxDataListLength@8
HidP_MaxUsageListLength@12
HidP_SetData@24
HidP_SetScaledUsageValue@32
HidP_SetUsageValue@32
HidP_SetUsageValueArray@36
HidP_SetUsages@32
HidP_SysPowerCaps@8
HidP_SysPowerEvent@16
HidP_TranslateUsageAndPagesToI8042ScanCodes@24
HidP_TranslateUsagesToI8042ScanCodes@24
HidP_UnsetUsages@32
HidP_UsageAndPageListDifference@20
HidP_UsageListDifference@20
