;
; Definition file of sppobjs.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "sppobjs.dll"
EXPORTS
SppPluginCanUnloadNow
SppPluginCreateInstance
SppPluginInitialize
SppPluginShutdown
SppPluginVersion
