LIBRARY DHCPCSVC.DLL
EXPORTS
DhcpAcquireParameters@4
DhcpAcquireParametersByBroadcast@4
DhcpCApiCleanup
DhcpCApiCleanup@0
DhcpCApiInitialize@4
DhcpClient_Generalize
DhcpDeRegisterConnectionStateNotification@8
DhcpDeRegisterOptions@4
DhcpDeRegisterParamChange@12
DhcpDelPersistentRequestParams@8
DhcpEnableDhcp@8
DhcpEnableDhcpAdvanced@24
DhcpEnableTracing@4
DhcpEnumClasses@16
DhcpEnumInterfaces@4
DhcpFallbackRefreshParams@4
DhcpFreeEnumeratedInterfaces@4
DhcpFreeLeaseInfo@4
DhcpFreeLeaseInfoArray@8
DhcpFreeMem@4
DhcpGetClassId@8
DhcpGetClientId@8
DhcpGetDhcpServicedConnections@12
DhcpGetFallbackParams@8
DhcpGetNotificationStatus@8
DhcpGetOriginalSubnetMask@8
DhcpGetTraceArray@4
DhcpGlobalIsShuttingDown DATA
DhcpGlobalServiceSyncEvent DATA
DhcpGlobalTerminateEvent DATA
DhcpHandlePnPEvent@20
DhcpIsEnabled@8
DhcpIsMeteredDetected@8
DhcpLeaseIpAddress@24
DhcpLeaseIpAddressEx@32
DhcpNotifyConfigChange@28
DhcpNotifyConfigChangeEx@32
DhcpNotifyMediaReconnected@4
DhcpOpenGlobalEvent
DhcpPersistentRequestParams@28
DhcpQueryLeaseInfo@8
DhcpQueryLeaseInfoArray@12
DhcpQueryLeaseInfoEx@12
DhcpRegisterConnectionStateNotification@12
DhcpRegisterOptions@16
DhcpRegisterParamChange@28
DhcpRemoveDNSRegistrations@0
DhcpReleaseIpAddressLease@8
DhcpReleaseIpAddressLeaseEx@16
DhcpReleaseParameters@4
DhcpRemoveDNSRegistrations
DhcpRenewIpAddressLease@16
DhcpRenewIpAddressLeaseEx@24
DhcpRequestCachedParams@20
DhcpRequestOptions@28
DhcpRequestParams@44
DhcpSetClassId@8
DhcpSetClientId@8
DhcpSetFallbackParams@8
DhcpSetMSFTVendorSpecificOptions@24
DhcpStaticRefreshParams@4
DhcpUndoRequestParams@16
Dhcpv4CheckServerAvailability@8
Dhcpv4EnableDhcpEx@4
McastApiCleanup
McastApiStartup@4
McastEnumerateScopes@20
McastGenUID@4
McastReleaseAddress@12
McastRenewAddress@16
McastRequestAddress@20
