;
; Definition file of SLC.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "SLC.dll"
EXPORTS
SLpCheckProductKey
SLpGetGenuineLocal
SLpProcessOemProductKey
SLpUpdateComponentTokens
SLClose
SLConsumeRight
SLConsumeWindowsRight
SLDepositOfflineConfirmationId
SLDepositOfflineConfirmationIdEx
SLFireEvent
SLGenerateOfflineInstallationId
SLGenerateOfflineInstallationIdEx
SLGetApplicationInformation
SLGetGenuineInformation
SLGetInstalledProductKeyIds
SLGetLicense
SLGetLicenseFileId
SLGetLicenseInformation
SLGetLicensingStatusInformation
SLGetPKeyId
SLGetPKeyInformation
SLGetPolicyInformation
SLGetPolicyInformationDWORD
SLGetProductSkuInformation
SLGetSLIDList
SLGetServiceInformation
SLGetWindowsInformation
SLGetWindowsInformationDWORD
SLInstallLicense
SLInstallProofOfPurchase
SLIsWindowsGenuineLocal
SLOpen
SLReArmWindows
SLRegisterEvent
SLRegisterWindowsEvent
SLSetCurrentProductKey
SLSetGenuineInformation
SLUninstallLicense
SLUninstallProofOfPurchase
SLUnregisterEvent
SLUnregisterWindowsEvent
