;
; Definition file of RASCHAPEXT.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "RASCHAPEXT.dll"
EXPORTS
RasChapExt_FreeMemory
RasChapExt_GetConfigForceNotDomainJoined
RasChapExt_GetConfigIgnoreIASLogon
RasChapExt_GetConfigKeepCredentialsOnFailure
RasChapExt_GetUserCredentials
RasChapExt_ShowHelp
