;
; Definition file of MDMRegistration.DLL
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "MDMRegistration.DLL"
EXPORTS
DiscoverManagementService
DiscoverManagementServiceEx
GetManagementAppHyperlink
IsDeviceRegisteredWithManagement
IsManagementRegistrationAllowed
RegisterDeviceWithManagement
SetManagedExternally
UnregisterDeviceWithManagement
