; 
; Exports of file dsprop.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY dsprop.dll
EXPORTS
CheckADsError
CrackName
DSPROP_GetGCSearchOnDomain
ErrMsg
ErrMsgParam
FindSheet
MsgBox
ReportError
Smart_PADS_ATTR_INFO__Empty
ADsPropCheckIfWritable
ADsPropCreateNotifyObj
ADsPropGetInitInfo
ADsPropSendErrorMessage
ADsPropSetHwnd
ADsPropSetHwndWithTitle
ADsPropShowErrorDialog
BringSheetToForeground
DllCanUnloadNow
DllGetClassObject
DllRegisterServer
DllUnregisterServer
IsSheetAlreadyUp
PostADsPropSheet
