;
; Definition file of srvcli.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "srvcli.dll"
EXPORTS
I_NetDfsGetVersion
I_NetServerSetServiceBits
I_NetServerSetServiceBitsEx
LocalAliasGet
LocalFileClose
LocalFileEnum
LocalFileEnumEx
LocalFileGetInfo
LocalFileGetInfoEx
LocalSessionDel
LocalSessionEnum
LocalSessionEnumEx
LocalSessionGetInfo
LocalSessionGetInfoEx
LocalShareAdd
LocalShareDelEx
LocalShareEnum
LocalShareEnumEx
LocalShareGetInfo
LocalShareGetInfoEx
LocalShareSetInfo
NetConnectionEnum
NetFileClose
NetFileEnum
NetFileGetInfo
NetRemoteTOD
NetServerAliasAdd
NetServerAliasDel
NetServerAliasEnum
NetServerComputerNameAdd
NetServerComputerNameDel
NetServerDiskEnum
NetServerGetInfo
NetServerSetInfo
NetServerStatisticsGet
NetServerTransportAdd
NetServerTransportAddEx
NetServerTransportDel
NetServerTransportEnum
NetSessionDel
NetSessionEnum
NetSessionGetInfo
NetShareAdd
NetShareCheck
NetShareDel
NetShareDelEx
NetShareDelSticky
NetShareEnum
NetShareEnumSticky
NetShareGetInfo
NetShareSetInfo
NetpsNameCanonicalize
NetpsNameCompare
NetpsNameValidate
NetpsPathCanonicalize
NetpsPathCompare
NetpsPathType
