;
; Definition file of rdvvmtransport.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "rdvvmtransport.dll"
EXPORTS
RdvTransport_CreateInstance
RdvTransport_GetInstance
RdvTransport_TerminateInstance
