LIBRARY api-ms-win-core-processthreads-l1-1-0

EXPORTS

CreateProcessA@40
CreateProcessAsUserW@44
CreateProcessW@40
CreateRemoteThread@28
CreateRemoteThreadEx@32
CreateThread@24
DeleteProcThreadAttributeList@4
ExitProcess@4
ExitThread@4
FlushProcessWriteBuffers@0
GetCurrentProcess@0
GetCurrentProcessId@0
GetCurrentThread@0
GetCurrentThreadId@0
GetExitCodeProcess@8
GetExitCodeThread@8
GetPriorityClass@4
GetProcessId@4
GetProcessIdOfThread@4
GetProcessTimes@20
GetProcessVersion@4
GetStartupInfoW@4
GetThreadId@4
GetThreadPriority@4
GetThreadPriorityBoost@8
InitializeProcThreadAttributeList@16
OpenProcessToken@12
OpenThread@12
OpenThreadToken@16
ProcessIdToSessionId@8
QueryProcessAffinityUpdateMode@8
QueueUserAPC@12
ResumeThread@4
SetPriorityClass@8
SetProcessAffinityUpdateMode@8
SetProcessShutdownParameters@8
SetThreadPriority@8
SetThreadPriorityBoost@8
SetThreadStackGuarantee@4
SetThreadToken@8
SuspendThread@4
SwitchToThread@0
TerminateProcess@8
TerminateThread@8
TlsAlloc@0
TlsFree@4
TlsGetValue@4
TlsSetValue@8
UpdateProcThreadAttribute@28
