;
; Definition file of TimeBrokerClient.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "TimeBrokerClient.dll"
EXPORTS
TbCreateCEvent
TbCreateEvent
TbDeleteCEvent
TbDeleteEvent
TbEnumerateCEvents
TbEnumerateEvents
TbQueryCEventData
TbQueryEventData
TbUpdateCEvent
TbUpdateEvent
