; 
; Exports of file catsrvut.DLL
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY catsrvut.DLL
EXPORTS
; public: __cdecl CComPlusComponent::CComPlusComponent(class CComPlusComponent const & __ptr64) __ptr64
??0CComPlusComponent@@QEAA@AEBV0@@Z
; public: __cdecl CComPlusInterface::CComPlusInterface(class CComPlusInterface const & __ptr64) __ptr64
??0CComPlusInterface@@QEAA@AEBV0@@Z
; public: __cdecl CComPlusMethod::CComPlusMethod(class CComPlusMethod const & __ptr64) __ptr64
??0CComPlusMethod@@QEAA@AEBV0@@Z
; public: __cdecl CComPlusObject::CComPlusObject(class CComPlusObject const & __ptr64) __ptr64
??0CComPlusObject@@QEAA@AEBV0@@Z
; public: virtual __cdecl CComPlusComponent::~CComPlusComponent(void) __ptr64
??1CComPlusComponent@@UEAA@XZ
; public: virtual __cdecl CComPlusInterface::~CComPlusInterface(void) __ptr64
??1CComPlusInterface@@UEAA@XZ
; public: class CComPlusComponent & __ptr64 __cdecl CComPlusComponent::operator=(class CComPlusComponent const & __ptr64) __ptr64
??4CComPlusComponent@@QEAAAEAV0@AEBV0@@Z
; public: class CComPlusInterface & __ptr64 __cdecl CComPlusInterface::operator=(class CComPlusInterface const & __ptr64) __ptr64
??4CComPlusInterface@@QEAAAEAV0@AEBV0@@Z
; public: class CComPlusMethod & __ptr64 __cdecl CComPlusMethod::operator=(class CComPlusMethod const & __ptr64) __ptr64
??4CComPlusMethod@@QEAAAEAV0@AEBV0@@Z
; public: class CComPlusObject & __ptr64 __cdecl CComPlusObject::operator=(class CComPlusObject const & __ptr64) __ptr64
??4CComPlusObject@@QEAAAEAV0@AEBV0@@Z
; public: class CComPlusTypelib & __ptr64 __cdecl CComPlusTypelib::operator=(class CComPlusTypelib const & __ptr64) __ptr64
??4CComPlusTypelib@@QEAAAEAV0@AEBV0@@Z
; const  CComPlusComponent::`vftable'
??_7CComPlusComponent@@6B@
; const  CComPlusInterface::`vftable'
??_7CComPlusInterface@@6B@
; const  CComPlusMethod::`vftable'
??_7CComPlusMethod@@6B@
; const  CComPlusObject::`vftable'
??_7CComPlusObject@@6B@
; public: struct ITypeLib * __ptr64 __cdecl CComPlusTypelib::GetITypeLib(void) __ptr64
?GetITypeLib@CComPlusTypelib@@QEAAPEAUITypeLib@@XZ
RegDBBackup
RegDBRestore
StartMTSTOCOM
WinlogonHandlePendingInfOperations
CGMIsAdministrator
COMPlusUninstallActionW
DllCanUnloadNow
DllGetClassObject
DllRegisterServer
DllUnregisterServer
FindAssemblyModulesW
ManagedRequestW
QueryUserDllW
RunMTSToCom
SysprepComplus
SysprepComplus2
