;
; Definition file of FVEAPI.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "FVEAPI.dll"
EXPORTS
InternalFveIsVolumeEncrypted
FveAddAuthMethodInformation
FveAddAuthMethodSid
FveApplyGroupPolicy
FveApplyNkpCertChanges
FveAttemptAutoUnlock
FveAuthElementFromPassPhraseW
FveAuthElementFromPinW
FveAuthElementFromRecoveryPasswordW
FveAuthElementGetKeyFileNameW
FveAuthElementReadExternalKeyW
FveAuthElementToRecoveryPasswordW
FveAuthElementWriteExternalKeyW
FveBackupRecoveryInformationToAD
FveBindDataVolume
FveCanStandardUsersChangePassphraseByProxy
FveCanStandardUsersChangePin
FveCheckPassphrasePolicy
FveCheckTpmCapability
FveClearUserFlags
FveCloseHandle
FveCloseVolume
FveCommitChanges
FveConversionDecrypt
FveConversionDecryptEx
FveConversionEncrypt
FveConversionEncryptEx
FveConversionEncryptPendingReboot
FveConversionEncryptPendingRebootEx
FveConversionPause
FveConversionResume
FveConversionStop
FveConversionStopEx
FveDecrementClearKeyCounter
FveDeleteAuthMethod
FveDisableDeviceLockoutState
FveDiscardChanges
FveDraCertPresentInRegistry
FveEnableRawAccess
FveEnableRawAccessEx
FveEnableRawAccessW
FveEraseDrive
FveFindFirstVolume
FveFindNextVolume
FveFlagsToProtectorType
FveGenerateNkpSessionKeys
FveGetAllowKeyExport
FveGetAuthMethodGuids
FveGetAuthMethodInformation
FveGetAuthMethodSid
FveGetAuthMethodSidInformation
FveGetClearKeyCounter
FveGetDataSet
FveGetDescriptionW
FveGetDeviceLockoutData
FveGetFipsAllowDisabled
FveGetFveMethod
FveGetFveMethodEDrv
FveGetIdentificationFieldW
FveGetIdentity
FveGetKeyPackage
FveGetSecureBootBindingState
FveGetStatus
FveGetStatusW
FveGetUserFlags
FveGetVolumeNameW
FveInitVolume
FveInitVolumeEx
FveInitializeDeviceEncryption
FveInitializeDeviceEncryption2
FveIsAnyDataVolumeBoundToOSVolume
FveIsBoundDataVolume
FveIsBoundDataVolumeToOSVolume
FveIsDeviceLockable
FveIsDeviceLockedOut
FveIsHardwareReadyForConversion
FveIsHybridVolume
FveIsHybridVolumeW
FveIsPassphraseCompatibleW
FveIsRecoveryPasswordGroupValidW
FveIsRecoveryPasswordValidW
FveIsSchemaExtInstalled
FveIsVolumeEncryptable
FveKeyManagement
FveLockDevice
FveLockVolume
FveLogRecoveryReason
FveNeedsDiscoveryVolumeUpdate
FveNotifyVolumeAfterFormat
FveOpenVolumeByHandle
FveOpenVolumeExW
FveOpenVolumeW
FveProtectorTypeToFlags
FveQuery
FveQueryDeviceEncryptionSupport
FveRevertVolume
FveServiceDiscoveryVolume
FveSetAllowKeyExport
FveSetDescriptionW
FveSetFipsAllowDisabled
FveSetFveMethod
FveSetIdentificationFieldW
FveSetUserFlags
FveSysClearUserFlags
FveSysCloseVolume
FveSysGetUserFlags
FveSysOpenVolumeW
FveSysSetUserFlags
FveUnbindAllDataVolumeFromOSVolume
FveUnbindDataVolume
FveUnlockVolume
FveUnlockVolumeAuthMethodSid
FveUnlockVolumeWithAccessMode
FveUpdateBandIdBcd
FveUpdateDeviceLockoutState
FveUpdateDeviceLockoutStateEx
FveUpdatePinW
FveUpgradeVolume
FveValidateDeviceLockoutState
FveValidateExistingPassphraseW
FveValidateExistingPinW
