;
; Definition file of QShvHost.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "QShvHost.dll"
EXPORTS
QuarCreateSession
QuarDestroySession
QuarFreeMemory
QuarInitialize
QuarSessionEvaluateClientMachineHealth
QuarSessionGetFixupServerList
QuarSessionGetId
QuarSessionGetMachineInventory
QuarSessionGetShvResultList
QuarSessionGetSoHResponse
QuarSessionSetNewQuarantineStatus
QuarUninitialize
