;
; Definition file of ESENT.dll
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "ESENT.dll"
EXPORTS
DebugExtensionInitialize@8
DebugExtensionNotify@12
DebugExtensionUninitialize@0
JetAddColumn@28
JetAddColumnA@28
JetAddColumnW@28
JetAttachDatabase2@16
JetAttachDatabase2A@16
JetAttachDatabase2W@16
JetAttachDatabase3@20
JetAttachDatabase3A@20
JetAttachDatabase3W@20
JetAttachDatabase@12
JetAttachDatabaseA@12
JetAttachDatabaseW@12
JetAttachDatabaseWithStreaming@24
JetAttachDatabaseWithStreamingA@24
JetAttachDatabaseWithStreamingW@24
JetBackup@12
JetBackupA@12
JetBackupInstance@16
JetBackupInstanceA@16
JetBackupInstanceW@16
JetBackupW@12
JetBeginDatabaseIncrementalReseed@12
JetBeginDatabaseIncrementalReseedA@12
JetBeginDatabaseIncrementalReseedW@12
JetBeginExternalBackup@4
JetBeginExternalBackupInstance@8
JetBeginSession@16
JetBeginSessionA@16
JetBeginSessionW@16
JetBeginSurrogateBackup@16
JetBeginTransaction2@8
JetBeginTransaction3@16
JetBeginTransaction@4
JetCloseDatabase@12
JetCloseFile@4
JetCloseFileInstance@8
JetCloseTable@8
JetCommitTransaction2@16
JetCommitTransaction@8
JetCompact@24
JetCompactA@24
JetCompactW@24
JetComputeStats@8
JetConfigureProcessForCrashDump@4
JetConsumeLogData@20
JetConvertDDL@20
JetConvertDDLA@20
JetConvertDDLW@20
JetCreateDatabase2@20
JetCreateDatabase2A@20
JetCreateDatabase2W@20
JetCreateDatabase3@24
JetCreateDatabase3A@24
JetCreateDatabase3W@24
JetCreateDatabase@20
JetCreateDatabaseA@20
JetCreateDatabaseW@20
JetCreateDatabaseWithStreaming@28
JetCreateDatabaseWithStreamingA@28
JetCreateDatabaseWithStreamingW@28
JetCreateEncryptionKey@16
JetCreateIndex2@16
JetCreateIndex2A@16
JetCreateIndex2W@16
JetCreateIndex3A@16
JetCreateIndex3W@16
JetCreateIndex4A@16
JetCreateIndex4W@16
JetCreateIndex@28
JetCreateIndexA@28
JetCreateIndexW@28
JetCreateInstance2@16
JetCreateInstance2A@16
JetCreateInstance2W@16
JetCreateInstance@8
JetCreateInstanceA@8
JetCreateInstanceW@8
JetCreateTable@24
JetCreateTableA@24
JetCreateTableColumnIndex2@12
JetCreateTableColumnIndex2A@12
JetCreateTableColumnIndex2W@12
JetCreateTableColumnIndex3A@12
JetCreateTableColumnIndex3W@12
JetCreateTableColumnIndex4A@12
JetCreateTableColumnIndex4W@12
JetCreateTableColumnIndex5A@12
JetCreateTableColumnIndex5W@12
JetCreateTableColumnIndex@12
JetCreateTableColumnIndexA@12
JetCreateTableColumnIndexW@12
JetCreateTableW@24
JetDBUtilities@4
JetDBUtilitiesA@4
JetDBUtilitiesW@4
JetDatabaseScan@24
JetDefragment2@28
JetDefragment2A@28
JetDefragment2W@28
JetDefragment3@32
JetDefragment3A@32
JetDefragment3W@32
JetDefragment@24
JetDefragmentA@24
JetDefragmentW@24
JetDelete@8
JetDeleteColumn2@16
JetDeleteColumn2A@16
JetDeleteColumn2W@16
JetDeleteColumn@12
JetDeleteColumnA@12
JetDeleteColumnW@12
JetDeleteIndex@12
JetDeleteIndexA@12
JetDeleteIndexW@12
JetDeleteTable@12
JetDeleteTableA@12
JetDeleteTableW@12
JetDetachDatabase2@12
JetDetachDatabase2A@12
JetDetachDatabase2W@12
JetDetachDatabase@8
JetDetachDatabaseA@8
JetDetachDatabaseW@8
JetDupCursor@16
JetDupSession@8
JetEnableMultiInstance@12
JetEnableMultiInstanceA@12
JetEnableMultiInstanceW@12
JetEndDatabaseIncrementalReseed@24
JetEndDatabaseIncrementalReseedA@24
JetEndDatabaseIncrementalReseedW@24
JetEndExternalBackup@0
JetEndExternalBackupInstance2@8
JetEndExternalBackupInstance@4
JetEndSession@8
JetEndSurrogateBackup@8
JetEnumerateColumns@40
JetEscrowUpdate@36
JetExternalRestore2@40
JetExternalRestore2A@40
JetExternalRestore2W@40
JetExternalRestore@32
JetExternalRestoreA@32
JetExternalRestoreW@32
JetFreeBuffer@4
JetGetAttachInfo@12
JetGetAttachInfoA@12
JetGetAttachInfoInstance@16
JetGetAttachInfoInstanceA@16
JetGetAttachInfoInstanceW@16
JetGetAttachInfoW@12
JetGetBookmark@20
JetGetColumnInfo@28
JetGetColumnInfoA@28
JetGetColumnInfoW@28
JetGetCounter@12
JetGetCurrentIndex@16
JetGetCurrentIndexA@16
JetGetCurrentIndexW@16
JetGetCursorInfo@20
JetGetDatabaseFileInfo@16
JetGetDatabaseFileInfoA@16
JetGetDatabaseFileInfoW@16
JetGetDatabaseInfo@20
JetGetDatabaseInfoA@20
JetGetDatabaseInfoW@20
JetGetDatabasePages@32
JetGetErrorInfoW@20
JetGetIndexInfo@28
JetGetIndexInfoA@28
JetGetIndexInfoW@28
JetGetInstanceInfo@8
JetGetInstanceInfoA@8
JetGetInstanceInfoW@8
JetGetInstanceMiscInfo@16
JetGetLS@16
JetGetLock@12
JetGetLogFileInfo@16
JetGetLogFileInfoA@16
JetGetLogFileInfoW@16
JetGetLogInfo@12
JetGetLogInfoA@12
JetGetLogInfoInstance2@20
JetGetLogInfoInstance2A@20
JetGetLogInfoInstance2W@20
JetGetLogInfoInstance@16
JetGetLogInfoInstanceA@16
JetGetLogInfoInstanceW@16
JetGetLogInfoW@12
JetGetMaxDatabaseSize@16
JetGetObjectInfo@32
JetGetObjectInfoA@32
JetGetObjectInfoW@32
JetGetPageInfo2@24
JetGetPageInfo@24
JetGetRecordPosition@16
JetGetRecordSize2@16
JetGetRecordSize@16
JetGetResourceParam@16
JetGetSecondaryIndexBookmark@36
JetGetSessionInfo@16
JetGetSessionParameter@20
JetGetSystemParameter@24
JetGetSystemParameterA@24
JetGetSystemParameterW@24
JetGetTableColumnInfo@24
JetGetTableColumnInfoA@24
JetGetTableColumnInfoW@24
JetGetTableIndexInfo@24
JetGetTableIndexInfoA@24
JetGetTableIndexInfoW@24
JetGetTableInfo@20
JetGetTableInfoA@20
JetGetTableInfoW@20
JetGetThreadStats@8
JetGetTruncateLogInfoInstance@16
JetGetTruncateLogInfoInstanceA@16
JetGetTruncateLogInfoInstanceW@16
JetGetVersion@8
JetGotoBookmark@16
JetGotoPosition@12
JetGotoSecondaryIndexBookmark@28
JetGrowDatabase@16
JetIdle@8
JetIndexRecordCount2@20
JetIndexRecordCount@16
JetInit2@8
JetInit3@12
JetInit3A@12
JetInit3W@12
JetInit4@12
JetInit4A@12
JetInit4W@12
JetInit@4
JetIntersectIndexes@20
JetMakeKey@20
JetMove@16
JetOSSnapshotAbort@8
JetOSSnapshotEnd@8
JetOSSnapshotFreeze@16
JetOSSnapshotFreezeA@16
JetOSSnapshotFreezeW@16
JetOSSnapshotGetFreezeInfo@16
JetOSSnapshotGetFreezeInfoA@16
JetOSSnapshotGetFreezeInfoW@16
JetOSSnapshotPrepare@8
JetOSSnapshotPrepareInstance@12
JetOSSnapshotThaw@8
JetOSSnapshotTruncateLog@8
JetOSSnapshotTruncateLogInstance@12
JetOnlinePatchDatabasePage@32
JetOpenDatabase@20
JetOpenDatabaseA@20
JetOpenDatabaseW@20
JetOpenFile@16
JetOpenFileA@16
JetOpenFileInstance@20
JetOpenFileInstanceA@20
JetOpenFileInstanceW@20
JetOpenFileSectionInstance@36
JetOpenFileSectionInstanceA@36
JetOpenFileSectionInstanceW@36
JetOpenFileW@16
JetOpenTable@28
JetOpenTableA@28
JetOpenTableW@28
JetOpenTempTable2@28
JetOpenTempTable3@28
JetOpenTempTable@24
JetOpenTemporaryTable2@8
JetOpenTemporaryTable@8
JetPatchDatabasePages@28
JetPatchDatabasePagesA@28
JetPatchDatabasePagesW@28
JetPrepareToCommitTransaction@16
JetPrepareUpdate@12
JetPrereadColumnsByReference@36
JetPrereadIndexRange@28
JetPrereadIndexRanges@32
JetPrereadKeys@28
JetPrereadTablesW@20
JetReadFile@16
JetReadFileInstance@20
JetRegisterCallback@24
JetRemoveLogfileA@12
JetRemoveLogfileW@12
JetRenameColumn@20
JetRenameColumnA@20
JetRenameColumnW@20
JetRenameTable@16
JetRenameTableA@16
JetRenameTableW@16
JetResetCounter@8
JetResetSessionContext@4
JetResetTableSequential@12
JetResizeDatabase@20
JetRestore2@12
JetRestore2A@12
JetRestore2W@12
JetRestore@8
JetRestoreA@8
JetRestoreInstance@16
JetRestoreInstanceA@16
JetRestoreInstanceW@16
JetRestoreW@8
JetRetrieveColumn@32
JetRetrieveColumnByReference@36
JetRetrieveColumnFromRecordStream@28
JetRetrieveColumns@16
JetRetrieveKey@24
JetRetrieveTaggedColumnList@28
JetRollback@8
JetSeek@12
JetSetColumn@28
JetSetColumnDefaultValue@28
JetSetColumnDefaultValueA@28
JetSetColumnDefaultValueW@28
JetSetColumns@16
JetSetCurrentIndex2@16
JetSetCurrentIndex2A@16
JetSetCurrentIndex2W@16
JetSetCurrentIndex3@20
JetSetCurrentIndex3A@20
JetSetCurrentIndex3W@20
JetSetCurrentIndex4@24
JetSetCurrentIndex4A@24
JetSetCurrentIndex4W@24
JetSetCurrentIndex@12
JetSetCurrentIndexA@12
JetSetCurrentIndexW@12
JetSetCursorFilter@20
JetSetDatabaseSize@16
JetSetDatabaseSizeA@16
JetSetDatabaseSizeW@16
JetSetIndexRange@12
JetSetLS@16
JetSetMaxDatabaseSize@16
JetSetResourceParam@16
JetSetSessionContext@8
JetSetSessionParameter@16
JetSetSystemParameter@20
JetSetSystemParameterA@20
JetSetSystemParameterW@20
JetSetTableInfo@20
JetSetTableInfoA@20
JetSetTableInfoW@20
JetSetTableSequential@12
JetSnapshotStart@12
JetSnapshotStartA@12
JetSnapshotStartW@12
JetSnapshotStop@8
JetStopBackup@0
JetStopBackupInstance@4
JetStopService@0
JetStopServiceInstance2@8
JetStopServiceInstance@4
JetStreamRecords@32
JetTerm2@8
JetTerm@4
JetTestHook@8
JetTracing@12
JetTruncateLog@0
JetTruncateLogInstance@4
JetUnregisterCallback@16
JetUpdate2@24
JetUpdate@20
JetUpgradeDatabase@16
JetUpgradeDatabaseA@16
JetUpgradeDatabaseW@16
ese@8
