; 
; Exports of file INETCFG.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY INETCFG.dll
EXPORTS
CheckConnectionWizard
ConfigureSystemForInternet
ConfigureSystemForInternetA
ConfigureSystemForInternetW
DllCanUnloadNow
DllGetClassObject
DllRegisterServer
DllUnregisterServer
FreeSignupWizard
InetConfigClient
InetConfigClientA
InetConfigClientW
InetConfigSystem
InetConfigSystemFromPath
InetConfigSystemFromPathA
InetConfigSystemFromPathW
InetGetAutodial
InetGetAutodialA
InetGetAutodialW
InetGetClientInfo
InetGetClientInfoA
InetGetClientInfoW
InetGetProxy
InetGetProxyA
InetGetProxyW
InetNeedModem
InetNeedSystemComponents
InetPerformSecurityCheck
InetSetAutodial
InetSetAutodialA
InetSetAutodialW
InetSetClientInfo
InetSetClientInfoA
InetSetClientInfoW
InetSetProxy
InetSetProxyA
InetSetProxyEx
InetSetProxyExA
InetSetProxyExW
InetSetProxyW
InetStartServices
IsSmartStart
IsSmartStartEx
LaunchSignupWizard
LaunchSignupWizardEx
SetAutoProxyConnectoid
SetInternetPhoneNumber
SetInternetPhoneNumberA
SetInternetPhoneNumberW
SetShellNext
SetShellNextA
SetShellNextW
_LaunchSignupWizardEx
