;
; Definition file of gpprnext.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "gpprnext.dll"
EXPORTS
PrinterGenerateGroupPolicy
PrinterProcessGroupPolicy
PrinterProcessGroupPolicyEx
