LIBRARY api-ms-win-shcore-registry-l1-1-0

EXPORTS

SHCopyKeyA
SHCopyKeyW
SHDeleteEmptyKeyA
SHDeleteEmptyKeyW
SHDeleteKeyA
SHDeleteKeyW
SHDeleteValueA
SHDeleteValueW
SHEnumKeyExA
SHEnumKeyExW
SHEnumValueA
SHEnumValueW
SHGetValueA
SHGetValueW
SHQueryInfoKeyA
SHQueryInfoKeyW
SHQueryValueExA
SHQueryValueExW
SHRegDuplicateHKey
SHRegGetIntW
SHRegGetPathA
SHRegGetPathW
SHRegGetValueA
SHRegGetValueW
SHRegSetPathA
SHRegSetPathW
SHSetValueA
SHSetValueW
