; 
; Exports of file wow64mib.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY wow64mib.dll
EXPORTS
SnmpExtensionClose
SnmpExtensionInit
SnmpExtensionInitEx
SnmpExtensionQuery
SnmpExtensionTrap
