;
; Definition file of osbaseln.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "osbaseln.dll"
EXPORTS
ord_1 @1
CloseOsBaseline
EnumOsBaselineComponentsA
EnumOsBaselineComponentsW
EnumOsOutOfDateComponentsA
EnumOsOutOfDateComponentsW
GetOsBaselineComponentInfoA
GetOsBaselineComponentInfoW
GetOsInstalledComponentInfoA
GetOsInstalledComponentInfoW
GetOsLatestBaselineServicePack
OpenOsBaseline
pGetOsBaselineCurrentVersion
pGetOsCurrentBaselineServicePack
pOpenOsBaselineByVersion
