;
; Definition file of clfsw32.dll
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "clfsw32.dll"
EXPORTS
LsnDecrement@4
AddLogContainer@16
AddLogContainerSet@20
AdvanceLogBase@16
AlignReservedLog@16
AllocReservedLog@12
CLFS_LSN_INVALID DATA
CLFS_LSN_NULL DATA
CloseAndResetLogFile@4
CreateLogContainerScanContext@24
CreateLogFile@24
CreateLogMarshallingArea@32
DeleteLogByHandle@4
DeleteLogFile@8
DeleteLogMarshallingArea@4
DeregisterManageableLogClient@4
DumpLogRecords@44
FlushLogBuffers@8
FlushLogToLsn@16
FreeReservedLog@12
GetLogContainerName@20
GetLogFileInformation@12
GetLogIoStatistics@20
GetNextLogArchiveExtent@16
HandleLogFull@4
InstallLogPolicy@8
LogTailAdvanceFailure@8
LsnBlockOffset@4
LsnContainer@4
LsnCreate@12
LsnEqual@8
LsnGreater@8
LsnIncrement@4
LsnInvalid@4
LsnLess@8
LsnNull@4
LsnRecordSequence@4
PrepareLogArchive@48
QueryLogPolicy@16
ReadLogArchiveMetadata@20
ReadLogNotification@12
ReadLogRecord@40
ReadLogRestartArea@24
ReadNextLogRecord@36
ReadPreviousLogRestartArea@20
RegisterForLogWriteNotification@12
RegisterManageableLogClient@8
RemoveLogContainer@16
RemoveLogContainerSet@20
RemoveLogPolicy@8
ReserveAndAppendLog@40
ReserveAndAppendLogAligned@44
ScanLogContainers@12
SetEndOfLog@12
SetLogArchiveMode@8
SetLogArchiveTail@12
SetLogFileSizeWithPolicy@12
TerminateLogArchive@4
TerminateReadLog@4
TruncateLog@12
ValidateLog@16
WriteLogRestartArea@32
