; 
; Exports of file OEMIGLIB.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY OEMIGLIB.dll
EXPORTS
OE5SimpleCreate
DllCanUnloadNow
DllGetClassObject
DllRegisterServer
DllUnregisterServer
