;
; Definition file of KDUSB.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "KDUSB.dll"
EXPORTS
KdD0Transition
KdD3Transition
KdDebuggerInitialize0
KdDebuggerInitialize1
KdReceivePacket
KdRestore
KdSave
KdSendPacket
KdSetHiberRange
