; 
; Exports of file INFOADMN.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY INFOADMN.dll
EXPORTS
CollectW3PerfData
FtpClearStatistics2
FtpQueryStatistics2
IISDisconnectUser
IISEnumerateUsers
InetInfoClearStatistics
InetInfoFlushMemoryCache
InetInfoGetAdminInformation
InetInfoGetGlobalAdminInformation
InetInfoGetServerCapabilities
InetInfoGetSites
InetInfoGetVersion
InetInfoQueryStatistics
InetInfoSetAdminInformation
InetInfoSetGlobalAdminInformation
InitW3CounterStructure
W3ClearStatistics2
W3QueryStatistics2
