;
; Definition file of SimPeerConfigDll.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "SimPeerConfigDll.dll"
EXPORTS
EapPeerGetIdentityPageGuid
InternalFunction01
InternalFunction02
EapPeerConfigBlob2Xml
EapPeerConfigXml2Blob
EapPeerCredentialsXml2Blob
EapPeerFreeErrorMemory
EapPeerFreeMemory
EapPeerGetConfigBlobAndUserBlob
EapPeerInvokeConfigUI
EapPeerInvokeIdentityUI
EapPeerInvokeInteractiveUI
