;
; Definition file of mibincodec.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "mibincodec.dll"
EXPORTS
MI_Application_NewDeserializer_Binary
MI_Application_NewSerializer_Binary
SyncBmilReader_Create
SyncBmilReader_Delete
SyncBmilReader_ReadInstance
SyncBmilWriter_Create
SyncBmilWriter_Delete
SyncBmilWriter_WriteInstance
