;
; Definition file of CSystemEventsBrokerClient.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "CSystemEventsBrokerClient.dll"
EXPORTS
CSebCreatePrivateEvent
CSebCreateWellKnownEvent
CSebDeleteEvent
CSebEnumerateEvents
CSebQueryEventData
