LIBRARY api-ms-win-core-sysinfo-l1-2-0

EXPORTS

EnumSystemFirmwareTables
GetNativeSystemInfo
GetOsSafeBootMode
GetProductInfo
GetSystemFirmwareTable
GetSystemTimePreciseAsFileTime
SetComputerNameExW
SetSystemTime
VerSetConditionMask
