LIBRARY scsiport.sys
EXPORTS
DllInitialize@4
ScsiDebugPrint
ScsiPortCompleteRequest@20
;ScsiPortConvertPhysicalAddressToUlong
ScsiPortConvertUlongToPhysicalAddress@4
ScsiPortFlushDma@4
ScsiPortFreeDeviceBase@8
ScsiPortGetBusData@24
ScsiPortGetDeviceBase@28
ScsiPortGetLogicalUnit@16
ScsiPortGetPhysicalAddress@16
ScsiPortGetSrb@20
ScsiPortGetUncachedExtension@12
ScsiPortGetVirtualAddress@12
ScsiPortInitialize@16
ScsiPortIoMapTransfer@16
ScsiPortLogError@28
ScsiPortMoveMemory@12
ScsiPortNotification
ScsiPortQuerySystemTime@4
ScsiPortReadPortBufferUchar@12
ScsiPortReadPortBufferUlong@12
ScsiPortReadPortBufferUshort@12
ScsiPortReadPortUchar@4
ScsiPortReadPortUlong@4
ScsiPortReadPortUshort@4
ScsiPortReadRegisterBufferUchar@12
ScsiPortReadRegisterBufferUlong@12
ScsiPortReadRegisterBufferUshort@12
ScsiPortReadRegisterUchar@4
ScsiPortReadRegisterUlong@4
ScsiPortReadRegisterUshort@4
ScsiPortSetBusDataByOffset@28
ScsiPortStallExecution@4
ScsiPortValidateRange@28
ScsiPortWritePortBufferUchar@12
ScsiPortWritePortBufferUlong@12
ScsiPortWritePortBufferUshort@12
ScsiPortWritePortUchar@8
ScsiPortWritePortUlong@8
ScsiPortWritePortUshort@8
ScsiPortWriteRegisterBufferUchar@12
ScsiPortWriteRegisterBufferUlong@12
ScsiPortWriteRegisterBufferUshort@12
ScsiPortWriteRegisterUchar@8
ScsiPortWriteRegisterUlong@8
ScsiPortWriteRegisterUshort@8
