;
; Definition file of OLEACC.dll
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "OLEACC.dll"
EXPORTS
DllRegisterServer@0
DllUnregisterServer@0
AccessibleChildren@20
AccessibleObjectFromEvent@20
AccessibleObjectFromPoint@16
AccessibleObjectFromWindow@16
CreateStdAccessibleObject@16
CreateStdAccessibleProxyA@20
CreateStdAccessibleProxyW@20
DllCanUnloadNow@0
DllGetClassObject@12
GetOleaccVersionInfo@8
GetProcessHandleFromHwnd@4
GetRoleTextA@12
GetRoleTextW@12
GetStateTextA@12
GetStateTextW@12
;IID_IAccessible DATA
;IID_IAccessibleHandler DATA
;LIBID_Accessibility DATA
LresultFromObject@12
ObjectFromLresult@16
PropMgrClient_LookupProp@28
WindowFromAccessibleObject@8
