;
; Definition file of WitnessWmiv2Provider.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "WitnessWmiv2Provider.dll"
EXPORTS
GetProviderClassID
MI_Main
WitnessWmiInitialize
WitnessWmiTerminate
