;
; Definition file of PLA.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "PLA.dll"
EXPORTS
ServiceMain
SvchostPushServiceGlobals
PlaDeleteReport
PlaExpandTaskArguments
PlaExtractCabinet
PlaGetLegacyAlertActionsFlagsFromString
PlaGetLegacyAlertActionsStringFromFlags
PlaGetServerCapabilities
PlaHost
PlaServer
PlaUpgrade
