LIBRARY api-ms-win-core-registry-l2-1-0

EXPORTS

RegConnectRegistryA
RegConnectRegistryW
RegCopyTreeA
RegCreateKeyA
RegCreateKeyTransactedA
RegCreateKeyTransactedW
RegCreateKeyW
RegDeleteKeyA
RegDeleteKeyTransactedA
RegDeleteKeyTransactedW
RegDeleteKeyW
RegDisablePredefinedCache
RegEnumKeyA
RegEnumKeyW
RegOpenKeyA
RegOpenKeyTransactedA
RegOpenKeyTransactedW
RegOpenKeyW
RegOverridePredefKey
RegQueryValueA
RegQueryValueW
RegReplaceKeyA
RegReplaceKeyW
RegSaveKeyA
RegSaveKeyW
RegSetValueA
RegSetValueW
