; 
; Exports of file MSDTCTM.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY MSDTCTM.dll
EXPORTS
DtcMainExt
ASCWrapObject
ASCDeliverDeferred
ASCDefer
ASCGetSafeReference
; public: static long __cdecl CUISCore::Create(class CUISCore * __ptr64 * __ptr64,struct IUnknown * __ptr64)
?Create@CUISCore@@SAJPEAPEAV1@PEAUIUnknown@@@Z
; public: static long __cdecl CTm::CreateInstance(class CTm * __ptr64 * __ptr64,struct IUnknown * __ptr64)
?CreateInstance@CTm@@SAJPEAPEAV1@PEAUIUnknown@@@Z
; public: static long __cdecl CXaTmCore::CreateInstance(class CXaTmCore * __ptr64 * __ptr64,struct IUnknown * __ptr64)
?CreateInstance@CXaTmCore@@SAJPEAPEAV1@PEAUIUnknown@@@Z
; long __cdecl CreateThreadPool(void)
?CreateThreadPool@@YAJXZ
ASCWrapClassFactory
DllGetClassObject
DllRegisterServer
DllUnregisterServer
GetTipFunctionalityWorking
SetTipFunctionalityWorking
