/* SPDX-License-Identifier: GPL-2.0 WITH Linux-syscall-note */
/*
 * Copyright (C) 2020-2022 Loongson Technology Corporation Limited
 */
#ifndef _ASM_SIGNAL_H
#define _ASM_SIGNAL_H

#define MINSIGSTKSZ 4096
#define SIGSTKSZ    16384

#include <asm-generic/signal.h>

#endif