/**
 * This file has no copyright assigned and is placed in the Public Domain.
 * This file is part of the mingw-w64 runtime package.
 * No warranty is given; refer to the file DISCLAIMER.PD within this package.
 */
#ifndef _INC_SPATIALAUDIOMETADATA
#define _INC_SPATIALAUDIOMETADATA

#define SPATIAL_AUDIO_POSITION 200
#define SPATIAL_AUDIO_POSITION_BYTE_COUNT (sizeof(float) * 3)
#define SPATIAL_AUDIO_STANDARD_COMMANDS_START 200

#endif /* _INC_SPATIALAUDIOMETADATA */
