; 
; Exports of file IPROP.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY IPROP.dll
EXPORTS
FmtIdToPropStgName
FreePropVariantArray
PropStgNameToFmtId
PropVariantClear
PropVariantCopy
StgCreatePropSetStg
StgCreatePropStg
StgOpenPropStg
