;
; Definition file of radardt.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "radardt.dll"
EXPORTS
RdrSysprepSpecialize
WdiDiagnosticModuleMain
WdiGetDiagnosticModuleInterfaceVersion
WdiHandleInstance
