; 
; Exports of file LogHours.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY LogHours.dll
EXPORTS
LogonScheduleDialog
ConnectionScheduleDialog
DialinHoursDialog
DirSyncScheduleDialog
LogonScheduleDialogEx
DialinHoursDialogEx
ReplicationScheduleDialog
ReplicationScheduleDialogEx
ConnectionScheduleDialogEx
DirSyncScheduleDialogEx
