LIBRARY api-ms-win-core-kernel32-legacy-l1-1-3

EXPORTS

CopyFileTransactedW@28
CreateDirectoryTransactedW@16
CreateHardLinkTransactedW@16
CreateMailslotW@16
CreateSymbolicLinkTransactedW@16
DeleteFileTransactedW@8
FindFirstFileTransactedW@28
GetBinaryTypeA@8
GetCompressedFileSizeTransactedW@12
GetFullPathNameTransactedW@20
GetLongPathNameTransactedW@16
MoveFileTransactedW@24
RemoveDirectoryTransactedW@8
SetFileAttributesTransactedW@12
