/*	$NetBSD: vhci.h,v 1.2 2020/03/31 16:34:25 maxv Exp $ */

/*
 * Copyright (c) 2019-2020 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Maxime Villard.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

typedef struct {
	int type;
#define VHCI_REQ_CTRL	0

	union {
		usb_device_request_t ctrl;
	} u;
} vhci_request_t;

typedef struct {
	size_t size;
} vhci_response_t;

struct vhci_ioc_get_info {
	/* General. */
	size_t nports;

	/* Current port. */
	u_int port;
	int status;

	/* Current addr. */
	uint8_t addr;
};

struct vhci_ioc_set_port {
	u_int port;
};

struct vhci_ioc_set_addr {
	uint8_t addr;
};

#define VHCI_IOC_GET_INFO	_IOR('V', 0, struct vhci_ioc_get_info)
#define VHCI_IOC_SET_PORT	_IOW('V', 1, struct vhci_ioc_set_port)
#define VHCI_IOC_SET_ADDR	_IOW('V', 2, struct vhci_ioc_set_addr)
#define VHCI_IOC_USB_ATTACH	_IO ('V', 10)
#define VHCI_IOC_USB_DETACH	_IO ('V', 11)