LIBRARY api-ms-win-security-base-l1-2-0

EXPORTS

AddResourceAttributeAce
AddScopedPolicyIDAce
CheckTokenCapability
CheckTokenMembershipEx
GetAppContainerAce
GetCachedSigningLevel
SetCachedSigningLevel
