;
; Definition file of ISCSIDSC.dll
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "ISCSIDSC.dll"
EXPORTS
AddISNSServerA
AddISNSServerW
AddIScsiConnectionA
AddIScsiConnectionW
AddIScsiSendTargetPortalA
AddIScsiSendTargetPortalW
AddIScsiStaticTargetA
AddIScsiStaticTargetW
AddPersistentIScsiDeviceA
AddPersistentIScsiDeviceW
ClearPersistentIScsiDevices
;DllMain
GetDevicesForIScsiSessionA
GetDevicesForIScsiSessionW
GetIScsiIKEInfoA
GetIScsiIKEInfoW
GetIScsiInitiatorNodeNameA
GetIScsiInitiatorNodeNameW
GetIScsiSessionListA
GetIScsiSessionListW
GetIScsiTargetInformationA
GetIScsiTargetInformationW
GetIScsiVersionInformation
LoginIScsiTargetA
LoginIScsiTargetW
LogoutIScsiTarget
RefreshISNSServerA
RefreshISNSServerW
RefreshIScsiSendTargetPortalA
RefreshIScsiSendTargetPortalW
RemoveISNSServerA
RemoveISNSServerW
RemoveIScsiConnection
RemoveIScsiPersistentTargetA
RemoveIScsiPersistentTargetW
RemoveIScsiSendTargetPortalA
RemoveIScsiSendTargetPortalW
RemoveIScsiStaticTargetA
RemoveIScsiStaticTargetW
RemovePersistentIScsiDeviceA
RemovePersistentIScsiDeviceW
ReportActiveIScsiTargetMappingsA
ReportActiveIScsiTargetMappingsW
ReportISNSServerListA
ReportISNSServerListW
ReportIScsiInitiatorListA
ReportIScsiInitiatorListW
ReportIScsiPersistentLoginsA
ReportIScsiPersistentLoginsW
ReportIScsiSendTargetPortalsA
ReportIScsiSendTargetPortalsExA
ReportIScsiSendTargetPortalsExW
ReportIScsiSendTargetPortalsW
ReportIScsiTargetPortalsA
ReportIScsiTargetPortalsW
ReportIScsiTargetsA
ReportIScsiTargetsW
ReportPersistentIScsiDevicesA
ReportPersistentIScsiDevicesW
SendScsiInquiry
SendScsiReadCapacity
SendScsiReportLuns
SetIScsiGroupPresharedKey
SetIScsiIKEInfoA
SetIScsiIKEInfoW
SetIScsiInitiatorCHAPSharedSecret
SetIScsiInitiatorNodeNameA
SetIScsiInitiatorNodeNameW
SetIScsiTunnelModeOuterAddressA
SetIScsiTunnelModeOuterAddressW
SetupPersistentIScsiDevices
SetupPersistentIScsiVolumes
