;
; Definition file of tsworkspace.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "tsworkspace.dll"
EXPORTS
RADCUISupportCreateSubscriptionClient
RADCUISupportCreateDiscoveryStrategy
RADCProcessGroupPolicyEx
TaskUpdateWorkspaces2
TaskUpdateWorkspaces
TaskUpdateWorkspacesIfNeeded
WorkspaceSilentSetupW
WorkspaceStatusNotify2
WorkspaceStatusNotify
