LIBRARY api-ms-win-core-psapi-l1-1-0

EXPORTS

K32EnumProcesses@12
K32GetModuleBaseNameW@16
K32GetModuleFileNameExW@16
K32GetModuleInformation@16
K32GetProcessMemoryInfo@12
K32QueryWorkingSet@12
K32QueryWorkingSetEx@12
QueryFullProcessImageNameW@16
