;
; Definition file of PSHED.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "PSHED.dll"
EXPORTS
PshedAllocateMemory
PshedArePluginsPresent
PshedAttemptErrorRecovery
PshedBugCheckSystem
PshedClearErrorRecord
PshedDisableErrorSource
PshedEnableErrorSource
PshedFinalizeErrorRecord
PshedFreeMemory
PshedGetAllErrorSources
PshedGetBootErrorPacket
PshedGetErrorSourceInfo
PshedGetInjectionCapabilities
PshedInitialize
PshedInjectError
PshedIsSystemWheaEnabled
PshedMarkHiberPhase
PshedReadErrorRecord
PshedRegisterPlugin
PshedRetrieveErrorInfo
PshedSetErrorSourceInfo
PshedSynchronizeExecution
PshedWriteErrorRecord
