; 
; Exports of file WLDAP32.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY WLDAP32.dll
EXPORTS
ldap_abandon
ldap_add
ldap_get_optionW
ldap_unbind
ldap_set_optionW
LdapGetLastError
cldap_open
LdapMapErrorToWin32
ldap_compare
ldap_delete
ldap_result2error
ldap_err2string
ldap_modify
ldap_modrdn
ldap_open
ldap_first_entry
ldap_next_entry
cldap_openW
LdapUTF8ToUnicode
ldap_get_dn
ldap_dn2ufn
ldap_first_attribute
ldap_next_attribute
ldap_get_values
ldap_get_values_len
ldap_count_entries
ldap_count_values
ldap_value_free
ldap_explode_dn
ldap_result
ldap_msgfree
ldap_addW
ldap_search
ldap_add_s
ldap_bind_s
ldap_unbind_s
ldap_delete_s
ldap_modify_s
ldap_modrdn_s
ldap_search_s
ldap_search_st
ldap_compare_s
LdapUnicodeToUTF8
ber_bvfree
cldap_openA
ldap_addA
ldap_add_ext
ldap_add_extA
ldap_simple_bind
ldap_simple_bind_s
ldap_bind
ldap_add_extW
ldap_add_ext_s
ldap_add_ext_sA
ldap_add_ext_sW
ldap_add_sA
ldap_modrdn2
ldap_modrdn2_s
ldap_add_sW
ldap_bindA
ldap_bindW
ldap_bind_sA
ldap_bind_sW
ldap_close_extended_op
ldap_compareA
ldap_compareW
ldap_count_values_len
ldap_compare_ext
ldap_value_free_len
ldap_compare_extA
ldap_compare_extW
ldap_perror
ldap_compare_ext_s
ldap_compare_ext_sA
ldap_compare_ext_sW
ldap_compare_sA
ldap_compare_sW
ldap_connect
ldap_control_free
ldap_control_freeA
ldap_control_freeW
ldap_controls_free
ldap_controls_freeA
ldap_controls_freeW
ldap_count_references
ldap_count_valuesA
ldap_count_valuesW
ldap_create_page_control
ldap_create_page_controlA
ldap_create_page_controlW
ldap_create_sort_control
ldap_create_sort_controlA
ldap_create_sort_controlW
ldap_deleteA
ldap_deleteW
ldap_delete_ext
ldap_delete_extA
ldap_delete_extW
ldap_delete_ext_s
ldap_delete_ext_sA
ldap_delete_ext_sW
ldap_delete_sA
ldap_delete_sW
ldap_dn2ufnW
ldap_encode_sort_controlA
ldap_encode_sort_controlW
ldap_err2stringA
ldap_err2stringW
ldap_escape_filter_elementA
ldap_escape_filter_elementW
ldap_explode_dnA
ldap_explode_dnW
ldap_extended_operation
ldap_extended_operationA
ldap_extended_operationW
ldap_first_attributeA
ldap_first_attributeW
ldap_first_reference
ldap_free_controls
ldap_free_controlsA
ldap_free_controlsW
ldap_get_dnA
ldap_get_dnW
ldap_get_next_page
ldap_get_next_page_s
ldap_get_option
ldap_get_optionA
ldap_get_paged_count
ldap_get_valuesA
ldap_get_valuesW
ldap_get_values_lenA
ldap_get_values_lenW
ldap_init
ldap_initA
ldap_initW
ldap_memfreeA
ldap_memfreeW
ldap_modifyA
ldap_modifyW
ldap_modify_ext
ldap_modify_extA
ldap_modify_extW
ldap_modify_ext_s
ldap_modify_ext_sA
ldap_modify_ext_sW
ldap_modify_sA
ldap_modify_sW
ldap_modrdn2A
ldap_modrdn2W
ldap_modrdn2_sA
ldap_modrdn2_sW
ldap_modrdnA
ldap_modrdnW
ldap_modrdn_sA
ldap_modrdn_sW
ldap_next_attributeA
ldap_next_attributeW
ldap_next_reference
ldap_openA
ldap_openW
ldap_parse_page_control
ldap_parse_page_controlA
ldap_parse_page_controlW
ldap_parse_reference
ldap_parse_referenceA
ldap_parse_referenceW
ldap_parse_result
ldap_parse_resultA
ldap_parse_resultW
ldap_parse_sort_control
ldap_parse_sort_controlA
ldap_parse_sort_controlW
ldap_rename_ext
ldap_rename_extA
ldap_rename_extW
ldap_rename_ext_s
ldap_rename_ext_sA
ldap_rename_ext_sW
ldap_searchA
ldap_searchW
ldap_search_abandon_page
ldap_search_ext
ldap_search_extA
ldap_search_extW
ldap_search_ext_s
ldap_search_ext_sA
ldap_escape_filter_element
ldap_set_dbg_flags
ldap_set_dbg_routine
ldap_memfree
ldap_startup
ldap_cleanup
ldap_search_ext_sW
ldap_search_init_page
ldap_search_init_pageA
ldap_search_init_pageW
ldap_search_sA
ldap_search_sW
ldap_search_stA
ldap_search_stW
ldap_set_option
ldap_set_optionA
ldap_simple_bindA
ldap_simple_bindW
ldap_simple_bind_sA
ldap_simple_bind_sW
ldap_sslinit
ldap_sslinitA
ldap_sslinitW
ldap_ufn2dn
ldap_ufn2dnA
ldap_ufn2dnW
ldap_value_freeA
ldap_value_freeW
ldap_check_filterA
ldap_check_filterW
ldap_dn2ufnA
ber_init
ber_free
ber_bvecfree
ber_bvdup
ber_alloc_t
ber_skip_tag
ber_peek_tag
ber_first_element
ber_next_element
ber_flatten
ber_printf
ber_scanf
ldap_conn_from_msg
ldap_sasl_bindW
ldap_sasl_bind_sW
ldap_sasl_bindA
ldap_sasl_bind_sA
ldap_parse_extended_resultW
ldap_parse_extended_resultA
ldap_create_vlv_controlW
ldap_create_vlv_controlA
ldap_parse_vlv_controlW
ldap_parse_vlv_controlA
ldap_start_tls_sW
ldap_start_tls_sA
ldap_stop_tls_s
ldap_extended_operation_sW
ldap_extended_operation_sA
