; 
; Exports of file sisbkup.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY sisbkup.dll
EXPORTS
SisCSFilesToBackupForLink
SisCreateBackupStructure
SisCreateRestoreStructure
SisFreeAllocatedMemory
SisFreeBackupStructure
SisFreeRestoreStructure
SisRestoredCommonStoreFile
SisRestoredLink
