LIBRARY api-ms-win-core-kernel32-legacy-l1-1-0

EXPORTS

AddLocalAlternateComputerNameW@8
BackupRead@28
BackupWrite@28
BindIoCompletionCallback@12
CopyFileA@12
CreateFileMappingA@24
CreateFileTransactedW@40
CreateMailslotA@16
CreateNamedPipeA@32
DnsHostnameToComputerNameW@12
DosDateTimeToFileTime@12
FileTimeToDosDateTime@12
FindResourceA@12
FindResourceExA@16
GetComputerNameA@8
GetComputerNameW@8
GetMaximumProcessorGroupCount@0
GetNamedPipeClientProcessId@8
GetNamedPipeServerProcessId@8
GetShortPathNameA@12
GetStartupInfoA@4
GetSystemPowerStatus@4
GetTapeParameters@16
GetThreadSelectorEntry@12
GlobalMemoryStatus@4
MoveFileA@8
MoveFileExA@12
MoveFileW@8
OpenFile@12
PulseEvent@4
RegisterWaitForSingleObject@24
SetFileCompletionNotificationModes@8
SetHandleCount@4
SetMailslotInfo@8
SetVolumeLabelW@8
UnregisterWait@4
WTSGetActiveConsoleSessionId@0
