; 
; Exports of file SHSVCS.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY SHSVCS.dll
EXPORTS
DllInstall
DllRegisterServer
DllUnregisterServer
HardwareDetectionServiceMain
ThemeServiceMain
CreateHardwareEventMoniker
DllCanUnloadNow
DllGetClassObject
