;
; Definition file of WDSCLIENTAPI.dll
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "WDSCLIENTAPI.dll"
EXPORTS
WdsCliAuthorizeSession
WdsCliCancelTransfer
WdsCliClose
WdsCliCreateSession
WdsCliFindFirstImage
WdsCliFindNextImage
WdsCliFreeDomainJoinInformation
WdsCliFreeStringArray
WdsCliFreeUnattendVariables
WdsCliGetClientUnattend
WdsCliGetDomainJoinInformation
WdsCliGetEnumerationFlags
WdsCliGetImageArchitecture
WdsCliGetImageDescription
WdsCliGetImageFiles
WdsCliGetImageGroup
WdsCliGetImageHalName
WdsCliGetImageHandleFromFindHandle
WdsCliGetImageHandleFromTransferHandle
WdsCliGetImageIndex
WdsCliGetImageLanguage
WdsCliGetImageLanguages
WdsCliGetImageLastModifiedTime
WdsCliGetImageName
WdsCliGetImageNamespace
WdsCliGetImageParameter
WdsCliGetImagePath
WdsCliGetImageSize
WdsCliGetImageType
WdsCliGetImageVersion
WdsCliGetTransferSize
WdsCliGetUnattendVariables
WdsCliInitializeLog
WdsCliLog
WdsCliObtainDriverPackages
WdsCliRegisterTrace
WdsCliTransferFile
WdsCliTransferImage
WdsCliWaitForTransfer
