LIBRARY api-ms-win-downlevel-advapi32-l1-1-0

EXPORTS

AccessCheck@32
AccessCheckAndAuditAlarmW@44
AccessCheckByType@44
AccessCheckByTypeAndAuditAlarmW@64
AccessCheckByTypeResultList@44
AccessCheckByTypeResultListAndAuditAlarmByHandleW@68
AccessCheckByTypeResultListAndAuditAlarmW@64
AddAccessAllowedAce@16
AddAccessAllowedAceEx@20
AddAccessAllowedObjectAce@28
AddAccessDeniedAce@16
AddAccessDeniedAceEx@20
AddAccessDeniedObjectAce@28
AddAce@20
AddAuditAccessAce@24
AddAuditAccessAceEx@28
AddAuditAccessObjectAce@36
AddMandatoryAce@20
AdjustTokenGroups@24
AdjustTokenPrivileges@24
AllocateAndInitializeSid@44
AllocateLocallyUniqueId@4
AreAllAccessesGranted@8
AreAnyAccessesGranted@8
CheckTokenMembership@12
ConvertToAutoInheritPrivateObjectSecurity@24
CopySid@12
CreatePrivateObjectSecurity@24
CreatePrivateObjectSecurityEx@32
CreatePrivateObjectSecurityWithMultipleInheritance@36
CreateProcessAsUserW@44
CreateRestrictedToken@36
CreateWellKnownSid@16
DeleteAce@8
DestroyPrivateObjectSecurity@4
DuplicateToken@12
DuplicateTokenEx@24
EqualDomainSid@12
EqualPrefixSid@8
EqualSid@8
EventActivityIdControl@8
EventEnabled@12
EventProviderEnabled@20
EventRegister@16
EventUnregister@8
EventWrite@20
EventWriteEx@40
EventWriteString@24
EventWriteTransfer@28
FindFirstFreeAce@8
FreeSid@4
GetAce@12
GetAclInformation@16
GetFileSecurityW@20
GetKernelObjectSecurity@20
GetLengthSid@4
GetPrivateObjectSecurity@20
GetSecurityDescriptorControl@12
GetSecurityDescriptorDacl@16
GetSecurityDescriptorGroup@12
GetSecurityDescriptorLength@4
GetSecurityDescriptorOwner@12
GetSecurityDescriptorRMControl@8
GetSecurityDescriptorSacl@16
GetSidIdentifierAuthority@4
GetSidLengthRequired@4
GetSidSubAuthority@8
GetSidSubAuthorityCount@4
GetTokenInformation@20
GetTraceEnableFlags@8
GetTraceEnableLevel@8
GetTraceLoggerHandle@4
GetWindowsAccountDomainSid@12
ImpersonateAnonymousToken@4
ImpersonateLoggedOnUser@4
ImpersonateNamedPipeClient@4
ImpersonateSelf@4
InitializeAcl@12
InitializeSecurityDescriptor@8
InitializeSid@12
IsTokenRestricted@4
IsValidAcl@4
IsValidSecurityDescriptor@4
IsValidSid@4
IsWellKnownSid@8
MakeAbsoluteSD@44
MakeSelfRelativeSD@12
MapGenericMask@8
ObjectCloseAuditAlarmW@12
ObjectDeleteAuditAlarmW@12
ObjectOpenAuditAlarmW@48
ObjectPrivilegeAuditAlarmW@24
OpenProcessToken@12
OpenThreadToken@16
PrivilegeCheck@12
PrivilegedServiceAuditAlarmW@20
QuerySecurityAccessMask@8
RegCloseKey@4
RegCopyTreeW@12
RegCreateKeyExA@36
RegCreateKeyExW@36
RegDeleteKeyExA@16
RegDeleteKeyExW@16
RegDeleteTreeA@8
RegDeleteTreeW@8
RegDeleteValueA@8
RegDeleteValueW@8
RegDisablePredefinedCacheEx@0
RegEnumKeyExA@32
RegEnumKeyExW@32
RegEnumValueA@32
RegEnumValueW@32
RegFlushKey@4
RegGetKeySecurity@16
RegGetValueA@28
RegGetValueW@28
RegLoadAppKeyA@20
RegLoadAppKeyW@20
RegLoadKeyA@12
RegLoadKeyW@12
RegLoadMUIStringA@28
RegLoadMUIStringW@28
RegNotifyChangeKeyValue@20
RegOpenCurrentUser@8
RegOpenKeyExA@20
RegOpenKeyExW@20
RegOpenUserClassesRoot@16
RegQueryInfoKeyA@48
RegQueryInfoKeyW@48
RegQueryValueExA@24
RegQueryValueExW@24
RegRestoreKeyA@12
RegRestoreKeyW@12
RegSaveKeyExA@16
RegSaveKeyExW@16
RegSetKeySecurity@12
RegSetValueExA@24
RegSetValueExW@24
RegUnLoadKeyA@8
RegUnLoadKeyW@8
RegisterTraceGuidsW@32
RevertToSelf@0
SetAclInformation@16
SetFileSecurityW@12
SetKernelObjectSecurity@12
SetPrivateObjectSecurity@20
SetPrivateObjectSecurityEx@24
SetSecurityAccessMask@8
SetSecurityDescriptorControl@12
SetSecurityDescriptorDacl@16
SetSecurityDescriptorGroup@12
SetSecurityDescriptorOwner@12
SetSecurityDescriptorRMControl@8
SetSecurityDescriptorSacl@16
SetThreadToken@8
SetTokenInformation@16
TraceEvent@12
TraceMessage
TraceMessageVa@24
UnregisterTraceGuids@8
