;
; Definition file of OLEACC.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "OLEACC.dll"
EXPORTS
AccGetRunningUtilityState
AccNotifyTouchInteraction
AccSetRunningUtilityState
AccessibleChildren
AccessibleObjectFromEvent
AccessibleObjectFromPoint
AccessibleObjectFromWindow
AccessibleObjectFromWindowTimeout
CreateStdAccessibleObject
CreateStdAccessibleProxyA
CreateStdAccessibleProxyW
GetOleaccVersionInfo
GetProcessHandleFromHwnd
GetRoleTextA
GetRoleTextW
GetStateTextA
GetStateTextW
IID_IAccessible
IID_IAccessibleHandler
LIBID_Accessibility
LresultFromObject
ObjectFromLresult
PropMgrClient_LookupProp
WindowFromAccessibleObject
