;
; Definition file of ShimEng.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "ShimEng.dll"
EXPORTS
SE_DllLoaded
SE_DllUnloaded
SE_DynamicShim
SE_GetHookAPIs
SE_GetMaxShimCount
SE_GetProcAddressIgnoreIncExc
SE_GetShimCount
SE_InstallAfterInit
SE_InstallBeforeInit
SE_IsShimDll
SE_ProcessDying
