;
; Definition file of schedcli.dll
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "schedcli.dll"
EXPORTS
NetScheduleJobAdd@12
NetScheduleJobDel@12
NetScheduleJobEnum@24
NetScheduleJobGetInfo@12
