;
; Definition file of d3dx11_43.dll
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "d3dx11_43.dll"
EXPORTS
D3DX11CheckVersion@8
D3DX11CompileFromFileA@44
D3DX11CompileFromFileW@44
D3DX11CompileFromMemory@52
D3DX11CompileFromResourceA@52
D3DX11CompileFromResourceW@52
D3DX11ComputeNormalMap@24
D3DX11CreateAsyncCompilerProcessor@40
D3DX11CreateAsyncFileLoaderA@8
D3DX11CreateAsyncFileLoaderW@8
D3DX11CreateAsyncMemoryLoader@12
D3DX11CreateAsyncResourceLoaderA@12
D3DX11CreateAsyncResourceLoaderW@12
D3DX11CreateAsyncShaderPreprocessProcessor@24
D3DX11CreateAsyncShaderResourceViewProcessor@12
D3DX11CreateAsyncTextureInfoProcessor@8
D3DX11CreateAsyncTextureProcessor@12
D3DX11CreateShaderResourceViewFromFileA@24
D3DX11CreateShaderResourceViewFromFileW@24
D3DX11CreateShaderResourceViewFromMemory@28
D3DX11CreateShaderResourceViewFromResourceA@28
D3DX11CreateShaderResourceViewFromResourceW@28
D3DX11CreateTextureFromFileA@24
D3DX11CreateTextureFromFileW@24
D3DX11CreateTextureFromMemory@28
D3DX11CreateTextureFromResourceA@28
D3DX11CreateTextureFromResourceW@28
D3DX11CreateThreadPump@12
D3DX11FilterTexture@16
D3DX11GetImageInfoFromFileA@16
D3DX11GetImageInfoFromFileW@16
D3DX11GetImageInfoFromMemory@20
D3DX11GetImageInfoFromResourceA@20
D3DX11GetImageInfoFromResourceW@20
D3DX11LoadTextureFromTexture@16
D3DX11PreprocessShaderFromFileA@28
D3DX11PreprocessShaderFromFileW@28
D3DX11PreprocessShaderFromMemory@36
D3DX11PreprocessShaderFromResourceA@36
D3DX11PreprocessShaderFromResourceW@36
D3DX11SHProjectCubeMap@24
D3DX11SaveTextureToFileA@16
D3DX11SaveTextureToFileW@16
D3DX11SaveTextureToMemory@20
