;
; Definition file of DEVMGR.DLL
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "DEVMGR.DLL"
EXPORTS
DeviceProperties_RunDLLA@16
DeviceProperties_RunDLLW@16
DevicePropertiesA@16
DevicePropertiesW@16
DeviceManager_ExecuteA@16
DeviceManager_ExecuteW@16
DeviceProblemTextA@20
DeviceProblemTextW@20
DeviceProblemWizardA@12
DeviceProblemWizardW@12
DeviceAdvancedPropertiesA@12
DeviceAdvancedPropertiesW@12
DeviceCreateHardwarePage@8
DeviceCreateHardwarePageEx@16
DevicePropertiesExA@20
DevicePropertiesExW@20
DeviceProblenWizard_RunDLLA@16
DeviceProblenWizard_RunDLLW@16
DeviceCreateHardwarePageCustom@20
