;
; Definition file of iesetup.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "iesetup.dll"
EXPORTS
IEApplyCurrentHardening
IEHardenAdmin
IEHardenAdminNow
IEHardenLMSettings
IEHardenMachineNow
IEHardenUser
IEShowHardeningDialog
