;
; Definition file of sxs.dll
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "sxs.dll"
EXPORTS
SxsFindClrClassInformation@24
SxsFindClrSurrogateInformation@24
SxsLookupClrGuid@24
SxsRunDllInstallAssembly@16
SxsRunDllInstallAssemblyW@16
SxspGenerateManifestPathOnAssemblyIdentity@16
CreateAssemblyCache@8
CreateAssemblyNameObject@16
SxsBeginAssemblyInstall@24
SxsEndAssemblyInstall@12
SxsGenerateActivationContext@4
SxsInstallW@4
SxsOleAut32MapConfiguredClsidToReferenceClsid@8
SxsOleAut32MapIIDOrCLSIDToTypeLibrary@8
SxsOleAut32MapIIDToProxyStubCLSID@8
SxsOleAut32MapIIDToTLBPath@16
SxsOleAut32MapReferenceClsidToConfiguredClsid@8
SxsOleAut32RedirectTypeLibrary@28
SxsProbeAssemblyInstallation@12
SxsQueryManifestInformation@28
SxsUninstallW@8
