;
; Definition file of Apphlpdm.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "Apphlpdm.dll"
EXPORTS
WdiDiagnosticModuleMain
WdiGetDiagnosticModuleInterfaceVersion
WdiHandleInstance
