;
; Definition file of WDSCORE.dll
; Automatic generated by gendef
; written by Kai Tietz 2008-2014
;
LIBRARY "WDSCORE.dll"
EXPORTS
??0?$CDynamicArray@EPAE@@QAA@I@Z
??0?$CDynamicArray@EPAUSKey@@@@QAA@I@Z
??0?$CDynamicArray@EPAUSValue@@@@QAA@I@Z
??0?$CDynamicArray@GPAG@@QAA@I@Z
??0?$CDynamicArray@PAUSEnumBinContext@@PAPAU1@@@QAA@I@Z
??0?$CDynamicArray@USKeeperEntry@CBlackboardFactory@@PAU12@@@QAA@I@Z
??0?$CDynamicArray@_KPA_K@@QAA@I@Z
??1?$CDynamicArray@EPAE@@QAA@XZ
??1?$CDynamicArray@EPAUSKey@@@@QAA@XZ
??1?$CDynamicArray@EPAUSValue@@@@QAA@XZ
??1?$CDynamicArray@GPAG@@QAA@XZ
??1?$CDynamicArray@PAUSEnumBinContext@@PAPAU1@@@QAA@XZ
??1?$CDynamicArray@USKeeperEntry@CBlackboardFactory@@PAU12@@@QAA@XZ
??1?$CDynamicArray@_KPA_K@@QAA@XZ
??4?$CDynamicArray@EPAE@@QAAAAV0@ABV0@@Z
??4?$CDynamicArray@EPAUSKey@@@@QAAAAV0@ABV0@@Z
??4?$CDynamicArray@EPAUSValue@@@@QAAAAV0@ABV0@@Z
??4?$CDynamicArray@GPAG@@QAAAAV0@ABV0@@Z
??4?$CDynamicArray@PAUSEnumBinContext@@PAPAU1@@@QAAAAV0@ABV0@@Z
??4?$CDynamicArray@USKeeperEntry@CBlackboardFactory@@PAU12@@@QAAAAV0@ABV0@@Z
??4?$CDynamicArray@_KPA_K@@QAAAAV0@ABV0@@Z
??A?$CDynamicArray@PAUSEnumBinContext@@PAPAU1@@@QAAAAPAUSEnumBinContext@@I@Z
??A?$CDynamicArray@_KPA_K@@QAAAA_KI@Z
??B?$CDynamicArray@EPAUSKey@@@@QBAPAUSKey@@XZ
??B?$CDynamicArray@EPAUSValue@@@@QBAPAUSValue@@XZ
??B?$CDynamicArray@GPAG@@QBAPAGXZ
??C?$CDynamicArray@EPAUSKey@@@@QBAPAUSKey@@XZ
??C?$CDynamicArray@EPAUSValue@@@@QBAPAUSValue@@XZ
??_F?$CDynamicArray@EPAE@@QAAXXZ
??_F?$CDynamicArray@EPAUSKey@@@@QAAXXZ
??_F?$CDynamicArray@EPAUSValue@@@@QAAXXZ
??_F?$CDynamicArray@GPAG@@QAAXXZ
??_F?$CDynamicArray@PAUSEnumBinContext@@PAPAU1@@@QAAXXZ
??_F?$CDynamicArray@USKeeperEntry@CBlackboardFactory@@PAU12@@@QAAXXZ
??_F?$CDynamicArray@_KPA_K@@QAAXXZ
?Add@?$CDynamicArray@PAUSEnumBinContext@@PAPAU1@@@QAAHAAPAUSEnumBinContext@@@Z
?Add@?$CDynamicArray@USKeeperEntry@CBlackboardFactory@@PAU12@@@QAAHAAUSKeeperEntry@CBlackboardFactory@@@Z
?Add@?$CDynamicArray@USKeeperEntry@CBlackboardFactory@@PAU12@@@QAAHAAUSKeeperEntry@CBlackboardFactory@@AAI@Z
?Add@?$CDynamicArray@_KPA_K@@QAAHAA_K@Z
?ElementAt@?$CDynamicArray@GPAG@@QAAAAGI@Z
?ElementAt@?$CDynamicArray@USKeeperEntry@CBlackboardFactory@@PAU12@@@QAAAAUSKeeperEntry@CBlackboardFactory@@I@Z
?GetBuffer@?$CDynamicArray@EPAE@@QAAPAEI@Z
?GetBuffer@?$CDynamicArray@EPAUSValue@@@@QAAPAUSValue@@I@Z
?GetBuffer@?$CDynamicArray@GPAG@@QAAPAGI@Z
?GetSize@?$CDynamicArray@EPAE@@QBAIXZ
?GetSize@?$CDynamicArray@GPAG@@QBAIXZ
?GetSize@?$CDynamicArray@PAUSEnumBinContext@@PAPAU1@@@QBAIXZ
?GetSize@?$CDynamicArray@USKeeperEntry@CBlackboardFactory@@PAU12@@@QBAIXZ
?GetSize@?$CDynamicArray@_KPA_K@@QBAIXZ
?Init@?$CDynamicArray@EPAE@@IAAXI@Z
?Init@?$CDynamicArray@EPAUSKey@@@@IAAXI@Z
?Init@?$CDynamicArray@EPAUSValue@@@@IAAXI@Z
?Init@?$CDynamicArray@GPAG@@IAAXI@Z
?Init@?$CDynamicArray@PAUSEnumBinContext@@PAPAU1@@@IAAXI@Z
?Init@?$CDynamicArray@USKeeperEntry@CBlackboardFactory@@PAU12@@@IAAXI@Z
?Init@?$CDynamicArray@_KPA_K@@IAAXI@Z
?RemoveAll@?$CDynamicArray@USKeeperEntry@CBlackboardFactory@@PAU12@@@QAAXXZ
?RemoveAll@?$CDynamicArray@_KPA_K@@QAAXXZ
?RemoveItemFromTail@?$CDynamicArray@PAUSEnumBinContext@@PAPAU1@@@QAAXXZ
?SetSize@?$CDynamicArray@EPAE@@QAAHK@Z
?SetSize@?$CDynamicArray@EPAUSKey@@@@QAAHK@Z
?SetSize@?$CDynamicArray@EPAUSValue@@@@QAAHK@Z
?SetSize@?$CDynamicArray@GPAG@@QAAHK@Z
?SetSize@?$CDynamicArray@PAUSEnumBinContext@@PAPAU1@@@QAAHK@Z
?SetSize@?$CDynamicArray@USKeeperEntry@CBlackboardFactory@@PAU12@@@QAAHK@Z
?SetSize@?$CDynamicArray@_KPA_K@@QAAHK@Z
WdsGetPointer
g_Kernel32 DATA
g_bEnableDiagnosticMode DATA
ConstructPartialMsgIfA
ConstructPartialMsgIfW
ConstructPartialMsgVA
ConstructPartialMsgVW
CurrentIP
EndMajorTask
EndMinorTask
GetMajorTask
GetMajorTaskA
GetMinorTask
GetMinorTaskA
StartMajorTask
StartMinorTask
WdsAbortBlackboardItemEnum
WdsAddModule
WdsAddUsmtLogStack
WdsAllocCollection
WdsCollectionAddValue
WdsCollectionGetValue
WdsCopyBlackboardItems
WdsCopyBlackboardItemsEx
WdsCreateBlackboard
WdsDeleteBlackboardValue
WdsDeleteEvent
WdsDestroyBlackboard
WdsDuplicateData
WdsEnableDiagnosticMode
WdsEnableExit
WdsEnableExitEx
WdsEnumFirstBlackboardItem
WdsEnumFirstCollectionValue
WdsEnumNextBlackboardItem
WdsEnumNextCollectionValue
WdsExecuteWorkQueue
WdsExecuteWorkQueue2
WdsExecuteWorkQueueEx
WdsExitImmediately
WdsExitImmediatelyEx
WdsFreeCollection
WdsFreeData
WdsGenericSetupLogInit
WdsGetAssertFlags
WdsGetBlackboardBinaryData
WdsGetBlackboardStringA
WdsGetBlackboardStringW
WdsGetBlackboardUintPtr
WdsGetBlackboardValue
WdsGetCurrentExecutionGroup
WdsGetSetupLog
WdsGetTempDir
WdsInitialize
WdsInitializeCallbackArray
WdsInitializeDataBinary
WdsInitializeDataStringA
WdsInitializeDataStringW
WdsInitializeDataUInt32
WdsInitializeDataUInt64
WdsIsDiagnosticModeEnabled
WdsIterateOfflineQueue
WdsIterateQueue
WdsLockBlackboardValue
WdsLockExecutionGroup
WdsLogCreate
WdsLogDestroy
WdsLogRegStockProviders
WdsLogRegisterProvider
WdsLogStructuredException
WdsLogUnRegStockProviders
WdsLogUnRegisterProvider
WdsPackCollection
WdsPublish
WdsPublishEx
WdsPublishImmediateAsync
WdsPublishImmediateEx
WdsPublishOffline
WdsSeqAlloc
WdsSeqFree
WdsSetAssertFlags
WdsSetBlackboardValue
WdsSetNextExecutionGroup
WdsSetUILanguage
WdsSetupLogDestroy
WdsSetupLogInit
WdsSetupLogMessageA
WdsSetupLogMessageW
WdsSubscribeEx
WdsTerminate
WdsUnlockExecutionGroup
WdsUnpackCollection
WdsUnsubscribe
WdsUnsubscribeEx
WdsValidBlackboard
