;
; Definition file of davclnt.dll
; Automatic generated by gendef
; written by Kai Tietz 2008
;
LIBRARY "davclnt.dll"
EXPORTS
DavCancelConnectionsToServer@8
DavFreeUsedDiskSpace@4
DavGetDiskSpaceUsage@16
DavGetTheLockOwnerOfTheFile@12
DavInvalidateCache@4
DavRegisterAuthCallback@8
DavUnregisterAuthCallback@4
DllCanUnloadNow@0
DllGetClassObject@12
NPAddConnection3@20
NPAddConnection@12
NPCancelConnection@8
NPCloseEnum@4
NPEnumResource@16
NPFormatNetworkName@20
NPGetCaps@4
NPGetConnection@12
NPGetResourceInformation@16
NPGetResourceParent@12
NPGetUniversalName@16
NPGetUser@12
NPOpenEnum@20
