; 
; Exports of file KEYMGR.dll
;
; Autogenerated by gen_exportdef
; Written by Kai Tietz, 2007
;
LIBRARY KEYMGR.dll
EXPORTS
CPlApplet
KRShowKeyMgr
PRShowRestoreFromMsginaW
PRShowRestoreWizardExW
PRShowRestoreWizardW
PRShowSaveFromMsginaW
PRShowSaveWizardExW
